@@ -1,6 +1,6 @@
 /*  Misfit Model 3D
  * 
- *  Copyright (c) 2004-2005 Kevin Worcester
+ *  Copyright (c) 2004-2007 Kevin Worcester
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -75,6 +75,13 @@ void Matrix::setTranslation( const double * vector )
    m_val[ 14 ] = vector[ 2 ];
 }
 
+void Matrix::setTranslation( const Vector & vector )
+{
+   m_val[ 12 ] = vector[ 0 ];
+   m_val[ 13 ] = vector[ 1 ];
+   m_val[ 14 ] = vector[ 2 ];
+}
+
 void Matrix::setTranslation( const double & x, const double & y, const double & z )
 {
    m_val[ 12 ] = x;
@@ -110,6 +117,47 @@ void Matrix::setRotation( const double * radians )
    }
 }
 
+void Matrix::setRotation( const Vector & radians )
+{
+   setRotation( radians.getVector() );
+}
+
+void Matrix::getRotation( double & x, double & y, double & z ) const
+{
+   double sinYaw;
+   double cosYaw;
+   double sinPitch = -m_val[2];
+   double cosPitch = sqrt( 1 - sinPitch*sinPitch );
+   double sinRoll;
+   double cosRoll;
+
+   // if cosPitch is NaN, it's because sinPitch^2 was slightly greater than 1.0
+   // in that case, cosPitch should be 0.0, so detect and correct it here
+   if ( cosPitch != cosPitch )
+   {
+      cosPitch = 0.0;
+   }
+
+   if ( fabs( cosPitch ) > 0.0001 )
+   {
+      sinRoll = m_val[6]  / cosPitch;
+      cosRoll = m_val[10] / cosPitch;
+      sinYaw  = m_val[1]  / cosPitch;
+      cosYaw  = m_val[0]  / cosPitch;
+   }
+   else
+   {
+      sinRoll = -m_val[9];
+      cosRoll = m_val[5];
+      sinYaw  = 0;
+      cosYaw  = 1;
+   }
+
+   z = atan2( sinYaw, cosYaw );
+   y = atan2( sinPitch, cosPitch );
+   x = atan2( sinRoll, cosRoll );
+}
+
 void Matrix::getRotation( double * radians ) const
 {
    if ( radians )
@@ -149,6 +197,18 @@ void Matrix::getRotation( double * radians ) const
    }
 }
 
+void Matrix::getRotation( Vector & radians ) const
+{
+   getRotation( radians.getVector() );
+}
+
+void Matrix::getTranslation( double & x, double & y, double & z ) const
+{
+   x = m_val[ 12 ];
+   y = m_val[ 13 ];
+   z = m_val[ 14 ];
+}
+
 void Matrix::getTranslation( double * vector ) const
 {
    if ( vector )
@@ -159,6 +219,13 @@ void Matrix::getTranslation( double * vector ) const
    }
 }
 
+void Matrix::getTranslation( Vector & vector ) const
+{
+   vector[ 0 ] = m_val[ 12 ];
+   vector[ 1 ] = m_val[ 13 ];
+   vector[ 2 ] = m_val[ 14 ];
+}
+
 void Matrix::setRotationInDegrees( const double * degrees )
 {
    double radVector[3];
@@ -168,6 +235,11 @@ void Matrix::setRotationInDegrees( const double * degrees )
    setRotation( radVector );
 }
 
+void Matrix::setRotationInDegrees( const Vector & degrees )
+{
+   setRotationInDegrees( degrees.getVector() );
+}
+
 void Matrix::setRotationInDegrees( double x, double y, double z )
 {
    double radVector[3];
@@ -223,7 +295,7 @@ void Matrix::setRotationOnAxis( const double * pVect, double radians )
    }
 }
 
-void Matrix::setRotationQuaternion( const Quaternion quat )
+void Matrix::setRotationQuaternion( const Quaternion & quat )
 {
    float xx      = quat.get(0) * quat.get(0);
    float xy      = quat.get(0) * quat.get(1);
@@ -253,6 +325,48 @@ void Matrix::setRotationQuaternion( const Quaternion quat )
    m_val[15] = 1;
 }
 
+void Matrix::getRotationQuaternion( Quaternion & quat )
+{
+   double s;
+
+   double t = 1 + m_val[0] + m_val[5] + m_val[10];
+   if ( t > 0.00001 )
+   {
+      s = sqrt(t) * 2;
+      quat.set( 0, ( m_val[6] - m_val[9] ) / s );
+      quat.set( 1, ( m_val[8] - m_val[2] ) / s );
+      quat.set( 2, ( m_val[1] - m_val[4] ) / s );
+      quat.set( 3, 0.25 * s );
+   }
+   else
+   {
+      if ( m_val[0] > m_val[5] && m_val[0] > m_val[10] ) // Column 0
+      {
+         s = sqrt( 1.0 + m_val[0] - m_val[5] - m_val[10] ) * 2;
+         quat.set( 0, 0.25 * s );
+         quat.set( 1, (m_val[1] + m_val[4] ) / s );
+         quat.set( 2, (m_val[8] + m_val[2] ) / s );
+         quat.set( 3, (m_val[6] - m_val[9] ) / s );
+      }
+      else if ( m_val[5] > m_val[10] ) // Column 1
+      {
+         s = sqrt( 1.0 + m_val[5] - m_val[0] - m_val[10] ) * 2;
+         quat.set( 0, (m_val[1] + m_val[4] ) / s );
+         quat.set( 1, 0.25 * s );
+         quat.set( 2, (m_val[6] + m_val[9] ) / s );
+         quat.set( 3, (m_val[8] - m_val[2] ) / s );
+      }
+      else // Column 2
+      {
+         s = sqrt( 1.0 + m_val[10] - m_val[0] - m_val[5] ) * 2;
+         quat.set( 0, (m_val[8] + m_val[2] ) / s );
+         quat.set( 1, (m_val[6] + m_val[9] ) / s );
+         quat.set( 2, 0.25 * s );
+         quat.set( 3, (m_val[1] - m_val[4] ) / s );
+      }
+   }
+}
+
 void Matrix::inverseRotateVector( double * pVect ) const
 {
     double vec[3];
@@ -271,6 +385,13 @@ void Matrix::inverseTranslateVector( double *pVect ) const
     pVect[2] = pVect[2] - m_val[14];
 }
 
+void Matrix::normalizeRotation()
+{
+   normalize3( &m_val[0] );
+   normalize3( &m_val[4] );
+   normalize3( &m_val[8] );
+}
+
 void Matrix::apply( float * pVec ) const
 {
    if ( pVec )
@@ -313,6 +434,11 @@ void Matrix::apply( double * pVec ) const
    }
 }
 
+void Matrix::apply( Vector & pVec ) const
+{
+   apply( pVec.getVector() );
+}
+
 void Matrix::apply3( float * pVec ) const
 {
    if ( pVec )
@@ -324,7 +450,6 @@ void Matrix::apply3( float * pVec ) const
             = pVec[ 0 ] * m_val[ ( 0<<2 ) + c ]
             + pVec[ 1 ] * m_val[ ( 1<<2 ) + c ]
             + pVec[ 2 ] * m_val[ ( 2<<2 ) + c ];
-//            + 1.0f      * m_val[ ( 3<<2 ) + c ];
       }
 
       pVec[0] = vec[0];
@@ -344,7 +469,6 @@ void Matrix::apply3( double * pVec ) const
             = pVec[ 0 ] * m_val[ ( 0<<2 ) + c ]
             + pVec[ 1 ] * m_val[ ( 1<<2 ) + c ]
             + pVec[ 2 ] * m_val[ ( 2<<2 ) + c ];
-//            + 1.0       * m_val[ ( 3<<2 ) + c ] ;
       }
 
       pVec[0] = vec[0];
@@ -353,6 +477,56 @@ void Matrix::apply3( double * pVec ) const
    }
 }
 
+void Matrix::apply3( Vector & pVec ) const
+{
+   apply3( pVec.getVector() );
+}
+
+void Matrix::apply3x( float * pVec ) const
+{
+   if ( pVec )
+   {
+      float vec[4];
+      for ( int c = 0; c < 3; c++ )
+      {
+         vec[ c ] 
+            = pVec[ 0 ] * m_val[ ( 0<<2 ) + c ]
+            + pVec[ 1 ] * m_val[ ( 1<<2 ) + c ]
+            + pVec[ 2 ] * m_val[ ( 2<<2 ) + c ]
+            + 1.0f      * m_val[ ( 3<<2 ) + c ];
+      }
+
+      pVec[0] = vec[0];
+      pVec[1] = vec[1];
+      pVec[2] = vec[2];
+   }
+}
+
+void Matrix::apply3x( double * pVec ) const
+{
+   if ( pVec )
+   {
+      double vec[3];
+      for ( int c = 0; c < 3; c++ )
+      {
+         vec[ c ] 
+            = pVec[ 0 ] * m_val[ ( 0<<2 ) + c ]
+            + pVec[ 1 ] * m_val[ ( 1<<2 ) + c ]
+            + pVec[ 2 ] * m_val[ ( 2<<2 ) + c ]
+            + 1.0       * m_val[ ( 3<<2 ) + c ];
+      }
+
+      pVec[0] = vec[0];
+      pVec[1] = vec[1];
+      pVec[2] = vec[2];
+   }
+}
+
+void Matrix::apply3x( Vector & pVec ) const
+{
+   apply3x( pVec.getVector() );
+}
+
 void Matrix::postMultiply( const Matrix & lhs )
 {
    double val[16];
@@ -417,17 +591,6 @@ Matrix Matrix::getSubMatrix( int i, int j ) const
       }
    }
 
-   // This shouldn't be necessary, ret should be 
-   // initialized to the identity matrix
-   /*
-   for ( int t = 0; t < 3; t++ )
-   {
-      ret.set( t, 3, 0.0 );
-      ret.set( 3, t, 0.0 );
-   }
-   ret.set( 3, 3, 1.0 );
-   */
-
    return ret;
 }
 
@@ -587,6 +750,22 @@ void Vector::set( int c, double val )
    m_val[ c ] = val;
 }
 
+void Vector::setAll( double x, double y, double z, double w )
+{
+   m_val[ 0 ] = x;
+   m_val[ 1 ] = y;
+   m_val[ 2 ] = z;
+   m_val[ 3 ] = w;
+}
+
+void Vector::setAll( const double * vec, int count )
+{
+   for ( int i = 0; i < count && i < 4; i++ )
+   {
+      m_val[ i ] = vec[i];
+   }
+}
+
 void Vector::show() const
 {
    for ( int c = 0; c < 4; c++ )
@@ -722,7 +901,7 @@ Vector Vector::operator-=( const Vector & rhs )
    return *this;
 }
 
-Quaternion::Quaternion( double * val )
+Quaternion::Quaternion( const double * val )
 {
    if ( val )
    {
@@ -740,6 +919,14 @@ Quaternion::Quaternion( double * val )
    }
 }
 
+Quaternion::Quaternion( const Vector & val )
+{
+   m_val[0] = val[0];
+   m_val[1] = val[1];
+   m_val[2] = val[2];
+   m_val[3] = val[3];
+}
+
 Quaternion::~Quaternion()
 {
 }
@@ -780,22 +967,43 @@ void Quaternion::setEulerAngles( const double * radians )
    axis[2] = 1.0;
    z.setRotationOnAxis( axis, radians[2] );
 
-   x = x * y;
-   x = x * z;
-
-   *this = x;
+   z = z * y;
+   z = z * x;
+   *this = z;
 }
 
 void Quaternion::setRotationOnAxis( const double * axis, double radians )
 {
-   if ( axis )
+   if ( axis && ::mag3( axis ) > 0.00001 )
    {
       for ( int t = 0; t < 3; t++ )
       {
          m_val[t] = axis[t];
       }
       m_val[3] = 0;
-      normalize();
+      normalize3();
+
+      double a = sin( radians / 2 );
+      double b = cos( radians / 2 );
+
+      m_val[0] *= a;
+      m_val[1] *= a;
+      m_val[2] *= a;
+      m_val[3]  = b;
+   }
+}
+
+void Quaternion::setRotationOnAxis( double x, double y, double z, double radians )
+{
+   double m = sqrt( x*x + y*y + z*z );
+   if ( m > 0.00001 )
+   {
+      m_val[0] = x;
+      m_val[1] = y;
+      m_val[2] = z;
+      m_val[3] = 0;
+
+      normalize3();
 
       double a = sin( radians / 2 );
       double b = cos( radians / 2 );
@@ -833,12 +1041,12 @@ void Quaternion::setRotationToPoint( const double & faceX, const double & faceY,
    float anglec = v1.get(0) * v2.get(0) + v1.get(1) * v2.get(1) + v1.get(2) * v2.get(2);
    double angle  = acos( anglec );
 
-// Plane equation: (given three points)
-//
-//     A = y1 (z2 - z3) + y2 (z3 - z1) + y3 (z1 - z2)
-//     B = z1 (x2 - x3) + z2 (x3 - x1) + z3 (x1 - x2)
-//     C = x1 (y2 - y3) + x2 (y3 - y1) + x3 (y1 - y2)
-//   - D = x1 (y2 z3 - y3 z2) + x2 (y3 z1 - y1 z3) + x3 (y1 z2 - y2 z1)
+   // Plane equation: (given three points)
+   //
+   //     A = y1 (z2 - z3) + y2 (z3 - z1) + y3 (z1 - z2)
+   //     B = z1 (x2 - x3) + z2 (x3 - x1) + z3 (x1 - x2)
+   //     C = x1 (y2 - y3) + x2 (y3 - y1) + x3 (y1 - y2)
+   //   - D = x1 (y2 z3 - y3 z2) + x2 (y3 z1 - y1 z3) + x3 (y1 z2 - y2 z1)
 
    float A = v1.get(1) * v2.get(2) - v2.get(1) * v1.get(2);
    float B = v1.get(2) * v2.get(0) - v2.get(3) * v1.get(0);
@@ -888,6 +1096,19 @@ void Quaternion::normalize()
    }
 }
 
+Quaternion Quaternion::swapHandedness()
+{
+   double axis[3] = { 0, 0, 0 };
+   double rad = 0;
+
+   this->getRotationOnAxis( axis, rad );
+
+   Quaternion rval;
+   rval.setRotationOnAxis( axis, -rad );
+
+   return rval;
+}
+
 double distance ( const Vector & v1, const Vector & v2 )
 {
    double xDiff = v2.get(0) - v1.get(0);
@@ -897,3 +1118,12 @@ double distance ( const Vector & v1, const Vector & v2 )
    return sqrt( xDiff*xDiff + yDiff*yDiff + zDiff*zDiff );
 }
 
+double distance ( const double * v1, const double * v2 )
+{
+   double xDiff = v2[0] - v1[0];
+   double yDiff = v2[1] - v1[1];
+   double zDiff = v2[2] - v1[2];
+
+   return sqrt( xDiff*xDiff + yDiff*yDiff + zDiff*zDiff );
+}
+
