@@ -1,6 +1,6 @@
 /*  Misfit Model 3D
  * 
- *  Copyright (c) 2004-2005 Kevin Worcester
+ *  Copyright (c) 2004-2007 Kevin Worcester
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "log.h"
 #include "texture.h"
 #include "texmgr.h"
+#include "translate.h"
 
 #ifdef MM3D_EDIT
 #include "modelundo.h"
@@ -33,7 +34,7 @@
 
 int Model::s_glTextures = 0;
 
-bool Model::loadTextures( unsigned context )
+bool Model::loadTextures( ContextT context )
 {
    LOG_PROFILE();
 
@@ -60,7 +61,9 @@ bool Model::loadTextures( unsigned context )
          if ( !tex )
          {
 #ifdef MM3D_EDIT
-            model_status( this, StatusError, STATUSTIME_LONG, "Could not load texture %s", m_materials[t]->m_filename.c_str() );
+            std::string msg = transll( QT_TRANSLATE_NOOP("LowLevel", "Could not load texture") );
+            msg += std::string(" ") + m_materials[t]->m_filename;
+            model_status( this, StatusError, STATUSTIME_LONG, msg.c_str() );
 #endif // MM3D_EDIT
             tex = TextureManager::getInstance()->getDefaultTexture( m_materials[t]->m_filename.c_str() );
          }
@@ -122,6 +125,7 @@ int Model::addTexture( Texture * tex )
       return -1;
    }
 
+   m_changeBits |= AddOther;
 
    if ( tex )
    {
@@ -172,6 +176,7 @@ int Model::addColorMaterial( const char * name )
       return -1;
    }
 
+   m_changeBits |= AddOther;
 
    int num = m_materials.size();
 
@@ -241,6 +246,8 @@ bool Model::setGroupTextureId( unsigned groupNumber, int textureId )
       return false;
    }
 
+   m_changeBits |= AddOther;
+
    if ( groupNumber >= 0 && groupNumber < m_groups.size() && textureId < (int) m_materials.size() )
    {
       m_validBspTree = false;
@@ -276,7 +283,7 @@ bool Model::setTextureCoords( const unsigned & triangleNumber, const unsigned &
       undo->addTextureCoords( triangleNumber, vertexIndex, s, t,
             m_triangles[ triangleNumber ]->m_s[ vertexIndex ], 
             m_triangles[ triangleNumber ]->m_t[ vertexIndex ] );
-      sendUndo( undo );
+      sendUndo( undo, true );
 
       m_triangles[ triangleNumber ]->m_s[ vertexIndex ] = s;
       m_triangles[ triangleNumber ]->m_t[ vertexIndex ] = t;
@@ -436,6 +443,53 @@ void Model::setTextureName( unsigned textureId, const char * name )
    }
 }
 
+void Model::setMaterialTexture( unsigned textureId, Texture * tex )
+{
+   if ( m_animationMode )
+   {
+      return;
+   }
+
+   if ( tex == NULL )
+   {
+      removeMaterialTexture( textureId );
+   }
+   else if ( textureId < m_materials.size() )
+   {
+      MU_SetMaterialTexture * undo = new MU_SetMaterialTexture();
+      undo->setMaterialTexture( textureId, tex, m_materials[ textureId ]->m_textureData );
+      sendUndo( undo );
+
+      m_materials[ textureId ]->m_textureData = tex;
+      m_materials[ textureId ]->m_filename = tex->m_filename;
+      m_materials[ textureId ]->m_type = Material::MATTYPE_TEXTURE;
+      invalidateTextures();
+   }
+}
+
+void Model::removeMaterialTexture( unsigned textureId )
+{
+   if ( m_animationMode )
+   {
+      return;
+   }
+
+   if ( textureId < m_materials.size() )
+   {
+      if ( m_materials[ textureId ]->m_type == Material::MATTYPE_TEXTURE )
+      {
+         MU_SetMaterialTexture * undo = new MU_SetMaterialTexture();
+         undo->setMaterialTexture( textureId, NULL, m_materials[ textureId ]->m_textureData );
+         sendUndo( undo );
+
+         m_materials[ textureId ]->m_textureData = NULL;
+         m_materials[ textureId ]->m_filename = "";
+         m_materials[ textureId ]->m_type = Material::MATTYPE_BLANK;
+         invalidateTextures();
+      }
+   }
+}
+
 bool Model::setTextureSClamp( unsigned textureId, bool clamp )
 {
    if ( textureId < m_materials.size() )
@@ -651,7 +705,7 @@ int Model::getMaterialColor( unsigned materialIndex, unsigned c, unsigned v )
    }
 }
 
-Model::Material::MaterialType Model::getMaterialType( unsigned materialIndex )
+Model::Material::MaterialTypeE Model::getMaterialType( unsigned materialIndex )
 {
    if ( materialIndex < m_materials.size() )
    {
@@ -682,7 +736,7 @@ int Model::getMaterialByName( const char * const materialName, bool ignoreCase )
    return matNumber;
 }
 
-DrawingContext * Model::getDrawingContext( unsigned context )
+DrawingContext * Model::getDrawingContext( ContextT context )
 {
    DrawingContextList::iterator it;
    for ( it = m_drawingContexts.begin(); it != m_drawingContexts.end(); it++ )
@@ -710,7 +764,7 @@ void Model::invalidateTextures()
    }
 }
 
-void Model::deleteGlTextures( unsigned context )
+void Model::deleteGlTextures( ContextT context )
 {
    DrawingContext * drawContext = getDrawingContext( context );
    for ( unsigned t = 0; t < drawContext->m_textures.size(); t++ )
@@ -726,7 +780,7 @@ void Model::deleteGlTextures( unsigned context )
    drawContext->m_valid = false;
 }
 
-void Model::removeContext( unsigned context )
+void Model::removeContext( ContextT context )
 {
    DrawingContext * drawContext = NULL;
    DrawingContextList::iterator it;
